package org.snarfed.android.headphonebuttons.bundle;

import android.os.Bundle;
import android.util.Log;

import org.snarfed.android.headphonebuttons.Constants;
import org.snarfed.android.headphonebuttons.ui.EditActivity;

/**
 * Class for managing the {@link com.twofortyfouram.locale.Intent#EXTRA_BUNDLE} for this plug-in.
 */
public final class PluginBundleManager
{
    /**
     * Private constructor prevents instantiation
     * 
     * @throws UnsupportedOperationException because this class cannot be instantiated.
     */
    private PluginBundleManager()
    {
        throw new UnsupportedOperationException("This class is non-instantiable"); //$NON-NLS-1$
    }

    /**
     * Type: {@code String}
     * <p>
     * Name of the headphone button to use.
     */
    public static final String BUNDLE_EXTRA_STRING_BUTTON = "org.snarfed.android.headphonebuttons.extra.STRING_BUTTON"; //$NON-NLS-1$

    /**
     * Type: {@code Integer}
     * <p>
     * Position of the selected item in the playing spinner. Corresponds to
     * the @array/playing_items resource.
     */
    public static final String BUNDLE_EXTRA_PLAYING_POSITION = "org.snarfed.android.headphonebuttons.extra.PLAYING_POSITION"; //$NON-NLS-1$

    /**
     * Method to verify the content of the bundle are correct.
     * <p>
     * This method will not mutate {@code bundle}.
     * 
     * @param bundle bundle to verify. May be null, which will always return false.
     * @return true if the Bundle is valid, false if the bundle is invalid.
     */
    public static boolean isBundleValid(final Bundle bundle)
    {
        if (null == bundle)
        {
            return false;
        }

        /*
         * Make sure the expected extras exist
         */
        if (!bundle.containsKey(BUNDLE_EXTRA_STRING_BUTTON) ||
            !bundle.containsKey(BUNDLE_EXTRA_PLAYING_POSITION))
        {
            if (Constants.IS_LOGGABLE)
            {
                Log.e(Constants.LOG_TAG, String.format(
                    "bundle must contain extras %s and %s",
                    BUNDLE_EXTRA_STRING_BUTTON, BUNDLE_EXTRA_PLAYING_POSITION)); //$NON-NLS-1$
            }
            return false;
        }

        /*
         * Make sure the correct number of extras exist. Run this test after checking for specific Bundle extras above so that the
         * error message is more useful. (E.g. the caller will see what extras are missing, rather than just a message that there
         * is the wrong number).
         */
        if (bundle.keySet().size() != 2)
        {
            if (Constants.IS_LOGGABLE)
            {
                Log.e(Constants.LOG_TAG, String.format("bundle contains extra keys: %s", bundle.keySet() //$NON-NLS-1$
                                                                                                                                                                       .toString()));
            }
            return false;
        }

        /*
         * Make sure the expected extras exist
         */
        String button = bundle.getString(BUNDLE_EXTRA_STRING_BUTTON);
        if (!EditActivity.BUTTONS.containsKey(button))
        {
            if (Constants.IS_LOGGABLE)
            {
                Log.e(Constants.LOG_TAG, String.format("Unknown or unsupported button value %s", button)); //$NON-NLS-1$
            }
            return false;
        }

        int pos = bundle.getInt(BUNDLE_EXTRA_PLAYING_POSITION);
        // note that the array length is hard coded. ugly.
        // i'd rather check against the real array length here, via:
        // context.getResources().getStringArray(R.array.playing_items)
        // but i don't have a context here. :/
        if (pos < 0 || pos >= 3)
        {
            if (Constants.IS_LOGGABLE)
            {
                Log.e(Constants.LOG_TAG, String.format("Unknown or unsupported plaing position value %s", pos)); //$NON-NLS-1$
            }
            return false;
        }

        return true;
    }
}
