package org.snarfed.android.headphonebuttons.ui;

import java.util.HashMap;

import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.ContextThemeWrapper;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.Spinner;
import android.widget.Toast;

import com.twofortyfouram.locale.BreadCrumber;
import org.snarfed.android.headphonebuttons.R;
import org.snarfed.android.headphonebuttons.bundle.BundleScrubber;
import org.snarfed.android.headphonebuttons.bundle.PluginBundleManager;

/**
 * This is the "Edit" activity for a Locale Plug-in.
 */
public final class EditActivity extends Activity
{
    public static final String TAG = "HeadphoneButtons";

    // these correspond to the items in the playing spinner in @array/playing_items.
    public static final int YES = 0;
    public static final int NO = 1;
    public static final int DOESNT_MATTER = 2;

    public static final HashMap<String, Integer> BUTTONS = new HashMap();
    static {
        BUTTONS.put("Headset button", KeyEvent.KEYCODE_HEADSETHOOK);
        BUTTONS.put("Play/Pause", KeyEvent.KEYCODE_MEDIA_PLAY_PAUSE);
        BUTTONS.put("Play", KeyEvent.KEYCODE_MEDIA_PLAY);
        BUTTONS.put("Pause", KeyEvent.KEYCODE_MEDIA_PAUSE);
        BUTTONS.put("Next", KeyEvent.KEYCODE_MEDIA_NEXT);
        BUTTONS.put("Previous", KeyEvent.KEYCODE_MEDIA_PREVIOUS);
        BUTTONS.put("Fast Forward", KeyEvent.KEYCODE_MEDIA_FAST_FORWARD);
        BUTTONS.put("Rewind", KeyEvent.KEYCODE_MEDIA_REWIND);
        BUTTONS.put("Stop", KeyEvent.KEYCODE_MEDIA_STOP);
    };

    /**
     * Help URL, used for the {@link com.twofortyfouram.locale.platform.R.id#twofortyfouram_locale_menu_help} menu item.
     */
    private static final String HELP_URL = "http://snarfed.org/locale_headphone_buttons_plugin"; //$NON-NLS-1$

    /**
     * Flag boolean that can only be set to true via the "Don't Save"
     * {@link com.twofortyfouram.locale.platform.R.id#twofortyfouram_locale_menu_dontsave} menu item in
     * {@link #onMenuItemSelected(int, MenuItem)}.
     * <p>
     * If true, then this {@code Activity} should return {@link Activity#RESULT_CANCELED} in {@link #finish()}.
     * <p>
     * If false, then this {@code Activity} should generally return {@link Activity#RESULT_OK} with extras
     * {@link com.twofortyfouram.locale.Intent#EXTRA_BUNDLE} and {@link com.twofortyfouram.locale.Intent#EXTRA_STRING_BLURB}.
     * <p>
     * There is no need to save/restore this field's state when the {@code Activity} is paused.
     */
    private boolean mIsCancelled = false;
    private String button;
    private RadioGroup buttonGroup;
    private Spinner playingSpinner;

    /**
     * {@inheritDoc}
     */
    @Override
    protected void onCreate(final Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);

        /*
         * A hack to prevent a private serializable classloader attack
         */
        BundleScrubber.scrub(getIntent());
        BundleScrubber.scrub(getIntent().getBundleExtra(com.twofortyfouram.locale.Intent.EXTRA_BUNDLE));
        BundleScrubber.scrub(savedInstanceState);

        setContentView(R.layout.main);
        setTitle(BreadCrumber.generateBreadcrumb(getApplicationContext(), getIntent(), getString(R.string.plugin_name)));

        final FrameLayout frame = (FrameLayout) findViewById(R.id.frame);
        ContextThemeWrapper themeWrapper = new ContextThemeWrapper(this, R.style.Theme_Locale_Light);
        frame.addView(getLayoutInflater().cloneInContext(themeWrapper).inflate(R.layout.frame, frame, false));

        if (savedInstanceState != null) {
            Log.d(TAG, "Trying to load savedInstanceState " + savedInstanceState);
            loadBundle(savedInstanceState);
        }

        playingSpinner = (Spinner) findViewById(R.id.playing_spinner);

        if (button == null) {
            final Bundle forwardedBundle = getIntent().getBundleExtra(com.twofortyfouram.locale.Intent.EXTRA_BUNDLE);
            Log.d(TAG, "Trying to load forwarded bundle " + forwardedBundle);
            loadBundle(forwardedBundle);
        }

        // populate radio buttons
        buttonGroup = (RadioGroup) findViewById(R.id.button_group);
        for (String label : BUTTONS.keySet()) {
            RadioButton radioButton = new RadioButton(themeWrapper);
            radioButton.setId(BUTTONS.get(label));
            radioButton.setText(label);
            radioButton.setOnClickListener(
                new View.OnClickListener() {
                    public void onClick(View v) {
                        button = ((RadioButton) v).getText().toString();
                    }
                });
            buttonGroup.addView(radioButton);
        }

        refreshDisplay();
    }

    /**
      * {@inheritDoc}
     */
    @Override
    public void onSaveInstanceState(Bundle bundle) {
        if (button != null) {
            bundle.putString(PluginBundleManager.BUNDLE_EXTRA_STRING_BUTTON, button);
            bundle.putInt(PluginBundleManager.BUNDLE_EXTRA_PLAYING_POSITION,
                          playingSpinner.getSelectedItemPosition());
        }
    }

    /** Tries to load a saved button from the given bundle. */
    public void loadBundle(Bundle bundle) {
        if (PluginBundleManager.isBundleValid(bundle)) {
            button = bundle.getString(PluginBundleManager.BUNDLE_EXTRA_STRING_BUTTON);
            playingSpinner.setSelection(bundle.getInt(
                PluginBundleManager.BUNDLE_EXTRA_PLAYING_POSITION));
        }
    }

    private void refreshDisplay() {
        if (button == null) {
            buttonGroup.clearCheck();
        } else {
            buttonGroup.check(BUTTONS.get(button));
        }
    }

    /**
      * {@inheritDoc}
     */
    @Override
    public void finish()
    {
        if (mIsCancelled)
        {
            setResult(RESULT_CANCELED);
        }
        else
        {
            /*
             * If there's nothing selected, then there isn't a setting to save.
             */
            if (button == null)
            {
                setResult(RESULT_CANCELED);
            }
            else
            {
                /*
                 * This is the return Intent, into which we'll put all the required extras
                 */
                final Intent returnIntent = new Intent();

                /*
                 * This extra is the data to ourselves: either for the Activity or the BroadcastReceiver. Note that anything
                 * placed in this Bundle must be available to Locale's class loader. So storing String, int, and other standard
                 * objects will work just fine. However Parcelable objects must also be Serializable. And Serializable objects
                 * must be standard Java objects (e.g. a private subclass to this plug-in cannot be stored in the Bundle, as
                 * Locale's classloader will not recognize it).
                 */
                final Bundle returnBundle = new Bundle();
                onSaveInstanceState(returnBundle);
                returnIntent.putExtra(com.twofortyfouram.locale.Intent.EXTRA_BUNDLE,
                                      returnBundle);

                /*
                 * This is the blurb concisely describing what your setting's state is. This is simply used for display in the UI.
                 */
                String[] playingBlurbs = {" (if playing)", " (if not playing)", ""};
                int playing = playingSpinner.getSelectedItemPosition();
                String blurb = button + playingBlurbs[playing];
                Log.d(TAG, blurb);

                if (blurb.length() > getResources().getInteger(R.integer.twofortyfouram_locale_maximum_blurb_length))
                {
                    returnIntent.putExtra(com.twofortyfouram.locale.Intent.EXTRA_STRING_BLURB, blurb.substring(0, getResources().getInteger(R.integer.twofortyfouram_locale_maximum_blurb_length)));
                }
                else
                {
                    returnIntent.putExtra(com.twofortyfouram.locale.Intent.EXTRA_STRING_BLURB, blurb);
                }

                setResult(RESULT_OK, returnIntent);
            }
        }

        super.finish();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean onCreateOptionsMenu(final Menu menu)
    {
        super.onCreateOptionsMenu(menu);

        /*
         * inflate the default menu layout from XML
         */
        getMenuInflater().inflate(R.menu.twofortyfouram_locale_help_save_dontsave, menu);

        return true;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean onMenuItemSelected(final int featureId, final MenuItem item)
    {

        /*
         * Royal pain in the butt to support the home button in SDK 11's ActionBar
         */
        if (Build.VERSION.SDK_INT >= 11)
        {
            try
            {
                if (item.getItemId() == android.R.id.class.getField("home").getInt(null)) //$NON-NLS-1$
                {
                    // app icon in Action Bar clicked; go home
                    final Intent intent = new Intent(getPackageManager().getLaunchIntentForPackage(getCallingPackage()));
                    intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                    startActivity(intent);
                    finish();
                    return true;
                }
            }
            catch (final NoSuchFieldException e)
            {
                // this should never happen on SDK 11 or greater
                throw new RuntimeException(e);
            }
            catch (final IllegalAccessException e)
            {
                // this should never happen on SDK 11 or greater
                throw new RuntimeException(e);
            }
        }

        switch (item.getItemId())
        {
            case R.id.twofortyfouram_locale_menu_help:
            {
                try
                {
                    startActivity(new Intent(Intent.ACTION_VIEW).setData(Uri.parse(HELP_URL)));
                }
                catch (final Exception e)
                {
                    Toast.makeText(getApplicationContext(), com.twofortyfouram.locale.platform.R.string.twofortyfouram_locale_application_not_available, Toast.LENGTH_LONG).show();
                }

                return true;
            }
            case R.id.twofortyfouram_locale_menu_dontsave:
            {
                mIsCancelled = true;
                finish();
                return true;
            }
            case R.id.twofortyfouram_locale_menu_save:
            {
                finish();
                return true;
            }
        }

        return super.onOptionsItemSelected(item);
    }
}
